export type JsonCodecTimestamp = [sessionId: number, time: number] | number;
export type JsonCodecTimespan = [sessionId: number, time: number, span: number];
export interface JsonCodecPatch {
    id: JsonCodecTimestamp;
    ops: JsonCodecOperation[];
}
export type JsonCodecOperation = JsonCodecMakeObjectOperation | JsonCodecMakeArrayOperation | JsonCodecMakeTupleOperation | JsonCodecMakeStringOperation | JsonCodecMakeBinaryOperation | JsonCodecMakeValueOperation | JsonCodecMakeConstantOperation | JsonCodecSetRootOperation | JsonCodecSetObjectKeysOperation | JsonCodecSetValueOperation | JsonCodecInsertStringSubstringOperation | JsonCodecBinaryInsertDataOperation | JsonCodecInsertArrayElementsOperation | JsonCodecDeleteOperation | JsonCodecNoopOperation;
export interface JsonCodecMakeObjectOperation {
    op: 'obj';
}
export interface JsonCodecMakeArrayOperation {
    op: 'arr';
}
export interface JsonCodecMakeTupleOperation {
    op: 'tup';
}
export interface JsonCodecMakeStringOperation {
    op: 'str';
}
export interface JsonCodecMakeBinaryOperation {
    op: 'bin';
}
export interface JsonCodecMakeValueOperation {
    op: 'val';
    value: JsonCodecTimestamp;
}
export interface JsonCodecMakeConstantOperation {
    op: 'const';
    timestamp?: boolean;
    value?: unknown | JsonCodecTimestamp;
}
export interface JsonCodecSetRootOperation {
    op: 'root';
    value: JsonCodecTimestamp;
}
export interface JsonCodecSetObjectKeysOperation {
    op: 'obj_set';
    obj: JsonCodecTimestamp;
    tuples: [key: string | number, value: JsonCodecTimestamp][];
}
export interface JsonCodecSetValueOperation {
    op: 'val_set';
    obj: JsonCodecTimestamp;
    value: JsonCodecTimestamp;
}
export interface JsonCodecInsertStringSubstringOperation {
    op: 'str_ins';
    obj: JsonCodecTimestamp;
    after?: JsonCodecTimestamp;
    value: string;
}
export interface JsonCodecBinaryInsertDataOperation {
    op: 'bin_ins';
    obj: JsonCodecTimestamp;
    after?: JsonCodecTimestamp;
    value: string;
}
export interface JsonCodecInsertArrayElementsOperation {
    op: 'arr_ins';
    obj: JsonCodecTimestamp;
    after?: JsonCodecTimestamp;
    values: JsonCodecTimestamp[];
}
export interface JsonCodecDeleteOperation {
    op: 'del';
    obj: JsonCodecTimestamp;
    what: JsonCodecTimespan[];
}
export interface JsonCodecNoopOperation {
    op: 'noop';
    len?: number;
}
